@@ -140,13 +140,14 @@ class BaseMessage(object):
 
     def dump_streams(self):
         if is_debug_active():
-            print("\nRequest :")
-            # hexdump( self._output_buffer.decode() )
-            print(repr(self._output_buffer))
-            print("\nResponse:")
-            # hexdump( self._input_buffer.decode() )
-            print(repr(self._input_buffer))
-            print("\n")
+            if len( self._output_buffer ):
+                print("\nRequest :")
+                hexdump( self._output_buffer )
+                # print(repr(self._output_buffer))
+            if len( self._input_buffer ):
+                print("\nResponse:")
+                hexdump( self._input_buffer )
+                # print(repr(self._input_buffer))
 
     def _append(self, field):
         """
@@ -165,6 +166,10 @@ class BaseMessage(object):
         if self._orientSocket.in_transaction is False:
             self._orientSocket.write( self._output_buffer )
             self._reset_fields_definition()
+        if is_debug_active():
+            self.dump_streams()
+            # reset output buffer
+            self._output_buffer = b""
 
         return self
 
@@ -197,18 +202,22 @@ class BaseMessage(object):
         elif t['type'] == BYTES:
             _content = struct.pack("!i", len(v)) + v
         elif t['type'] == STRING:
-            buf = v.encode('utf-8')
-            _content = struct.pack("!i", len(buf)) + buf
+            if sys.version_info.major >= 3:
+                if isinstance( v, str ):
+                    v = v.encode('utf-8')
+            _content = struct.pack("!i", len(v)) + v
         elif t['type'] == STRINGS:
             _content = b''
             for s in v:
-                buf = s.encode('utf-8')
-                _content += struct.pack("!i", len(buf)) + buf
+                if sys.version_info.major >= 3:
+                    if isinstance( s, str ):
+                        s = s.encode('utf-8')
+                _content += struct.pack("!i", len(s)) + s
 
         return _content
 
     def _decode_field(self, _type):
-        _value = ""
+        _value = b""
         # read buffer length and decode value by field definition
         if _type['bytes'] is not None:
             _value = self._orientSocket.read( _type['bytes'] )
@@ -219,7 +228,7 @@ class BaseMessage(object):
 
             _len = struct.unpack('!i', _value)[0]
             if _len == -1:
-                _decoded_string = ''
+                _decoded_string = b''
             else:
                 _decoded_string = self._orientSocket.read( _len )
 
