@@ -86,7 +86,7 @@ class OrientSocket(object):
         except socket.timeout as e:
             # we don't set false because of
             # RecordUpdateMessage/RecordCreateMessage trick
-            """:see RecordUpdateMessage.py """
+            """@see pyorient.messages.records.RecordUpdateMessage"""
             raise e
         except Exception as e:
             self._connected = False
@@ -113,7 +113,7 @@ def ByteToHex( byte_str ):
 #
 # OrientDB Message Factory
 #
-class OrientDB():
+class OrientDB(object):
     _connection = None
 
     _Messages = dict(
